using System.ComponentModel;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

namespace EQIA
{
    public delegate void XmlDocumentChanged(XmlDocument xmlDocument);

    /// <summary>
    /// The base type for an object that can be serialized to and from an xml document.
    /// </summary>
    public class EQIA_Object
    {
        #region Fields

        /// <summary>
        /// The amount of white space to indent xml blocks in the xml document. This setting is global.
        /// </summary>
        [XmlIgnore()]
        [Browsable(true), Description("The amount of white space to indent xml blocks in the xml document. This setting is global.")]
        public static int XmlIndentation = 5;

        private XmlDocument myXmlDocument;
        private bool isErrorInXml;

        #endregion

        /// <summary>
        /// If there were any errors in serializing the object, this will be set to true.
        /// </summary>
        [XmlIgnore(), Browsable(false)]
        public bool ErrorInXml { get { return isErrorInXml; } }

        /// <summary>
        /// Event fired whenever the xml document is changed.
        /// </summary>
        public event XmlDocumentChanged XmlDocumentChanged;

        #region Constructor

        /// <summary>
        /// Create a new object.
        /// </summary>
        public EQIA_Object() 
        {
            myXmlDocument = new XmlDocument();
            isErrorInXml = false;
        }

        #endregion

        #region Xml Methods

        /// <summary>
        /// Load this object's xml document.
        /// </summary>
        /// <param name="xml">The xml code for this object.</param>
        public void LoadXml(string xml)
        {
            myXmlDocument.LoadXml(xml);
        }

        /// <summary>
        /// Get the xml code for this object as a formatted string.
        /// </summary>
        /// <returns>A formatted string of this object's xml code.</returns>
        public string XmlToString()
        {
            StringWriter stringWriter = new StringWriter();
            using (XmlTextWriter xmlTextWriter = new XmlTextWriter(stringWriter))
            {
                xmlTextWriter.Formatting = Formatting.Indented;
                xmlTextWriter.Indentation = XmlIndentation;
                myXmlDocument.DocumentElement.WriteTo(xmlTextWriter);
            }
            return stringWriter.ToString();
        }

        /// <summary>
        /// Get the xml code in an xml document as a formatted string, this method will use the
        /// static indentation amount stored in this class.
        /// </summary>
        /// <param name="xmlDocument">The xml document to draw the code from.</param>
        /// <returns>A string containing the xml document as formatted xml mark-up.</returns>
        public static string XmlToString(XmlDocument xmlDocument)
        {
            return XmlToString(xmlDocument, XmlIndentation);
        }

        /// <summary>
        /// Get the xml code in an xml document as a formatted string.
        /// </summary>
        /// <param name="xmlDocument">The xml document to draw the code from.</param>
        /// <param name="indentation">The amount of white-spaces to indent new element blocks.</param>
        /// <returns>A string containing the xml document as formatted xml mark-up.</returns>
        public static string XmlToString(XmlDocument xmlDocument, int indentation)
        {
            StringWriter stringWriter = new StringWriter();
            using (XmlTextWriter xmlTextWriter = new XmlTextWriter(stringWriter))
            {
                xmlTextWriter.Formatting = Formatting.Indented;
                xmlTextWriter.Indentation = indentation;
                xmlDocument.DocumentElement.WriteTo(xmlTextWriter);
            }
            return stringWriter.ToString();
        }

        /// <summary>
        /// Deserialize this object from it's xml document.
        /// </summary>
        public void Deserialize()
        {
            XmlSerializer xmlSerializer = new XmlSerializer(this.GetType());
            using (XmlNodeReader xmlNodeReader = new XmlNodeReader(myXmlDocument.DocumentElement))
            {
                try
                {
                    object obj = xmlSerializer.Deserialize(xmlNodeReader);
                    isErrorInXml = false;
                    SetProperties(obj);
                }
                catch { isErrorInXml = true; }
            }
        }

        /// <summary>
        /// Serialize this object into it's xml document.
        /// </summary>
        public void Serialize()
        {
            StringWriter stringWriter = new StringWriter();
            using (XmlTextWriter xmlTextWriter = new XmlTextWriter(stringWriter))
            {
                XmlSerializer xmlSerializer = new XmlSerializer(this.GetType());
                XmlSerializerNamespaces xmlns = new XmlSerializerNamespaces();
                xmlns.Add("", "");
                xmlSerializer.Serialize(xmlTextWriter, this, xmlns);
            }

            myXmlDocument = new XmlDocument();
            myXmlDocument.LoadXml(stringWriter.ToString());

            if (XmlDocumentChanged != null)
                XmlDocumentChanged.Invoke(myXmlDocument);
        }

        /// <summary>
        /// Set the properties of this object from a deserialized object.
        /// </summary>
        /// <param name="obj">An object that contains the data from deserialization.</param>
        protected virtual void SetProperties(object obj) 
        { 
        }

        #endregion
    }
}
